import 'package:flutter/material.dart';

class ScrollerResizer extends StatefulWidget {
  final double windowWidth;
  final double childWidth;
  final double scale;
  final List<Widget> childs;
  final int select;

  const ScrollerResizer({Key? key, required this.windowWidth, required this.childWidth,
    this.scale = 0.7, required this.childs, this.select = 0}) : super(key: key);

  @override
  _ScrollerResizerState createState() => _ScrollerResizerState();
}

class _ScrollerResizerState extends State<ScrollerResizer> {

  List<Widget> w = [];
  List<ScaleData> widgets = [];

  final ScrollController _scrollController = ScrollController();

  double _scrollWidth = 0;

  @override
  void initState() {

    _scrollWidth = widget.windowWidth/2-widget.childWidth/2;

    for (var item in widget.childs) {
      if (widget.select == widget.childs.indexOf(item))
        widgets.add(ScaleData(1, item));
      else
        widgets.add(ScaleData(widget.scale, item));
    }

    if (widget.select != 0 && widget.select < widgets.length){
      var def = widget.childWidth*widget.select;
      WidgetsBinding.instance!.addPostFrameCallback((_) {
        _scrollController.jumpTo(def);
      });
    }

    _buildList();

    _scrollController.addListener((){
      var _scrollPosition = _scrollController.position.pixels;
      // print("_scrollPosition=$_scrollPosition");

      var v = (1-widget.scale)*100;
      var p1 = widget.childWidth/v;
      var p = _scrollPosition/p1/100;

      var t1 = 0;
      var t2 = 1;
      int k = _scrollPosition~/(widget.childWidth);
      var sp = _scrollPosition-(widget.childWidth*k);
      p = sp/p1/100;
      if  (k < widgets.length-1){
        t1 = k;
        t2 = k+1;
      }

      if (t1 > 0 && t1 < widgets.length) {
        var m = 1;
        do {
          widgets[t1 - m].scale = widget.scale;
          m++;
        } while (t1 - m >= 0);
      }

      widgets[t1].scale = 1-p;
      if (widgets[t1].scale < widget.scale) widgets[t1].scale = widget.scale;

      widgets[t2].scale = widget.scale+p;
      if (widgets[t2].scale > 1) widgets[t2].scale = 1;

      // print("_scrollPosition=$_scrollPosition windowWidth=${widget.windowWidth} k=$k");

      _buildList();
      _redraw();
    });

    super.initState();
  }

  @override
  void dispose() {
    _scrollController.dispose();
    super.dispose();
  }

  _buildList(){
    w = [];
    w.add(SizedBox(width: _scrollWidth));

    for (var item in widgets){
      w.add(Transform.scale(
          scale: item.scale,
          child: item.widget));
    }
    w.add(SizedBox(width: _scrollWidth));
  }

  _redraw(){
    if (mounted)
      setState(() {
      });
  }

  @override
  Widget build(BuildContext context) {
    return SizedBox(width: widget.windowWidth,
        child: Column(
          mainAxisSize: MainAxisSize.min,
          children: [
            SingleChildScrollView(
                controller: _scrollController,
                scrollDirection: Axis.horizontal,
                child: Row(
                    children: w
                ))
          ],
        ));
  }
}

class ScaleData{
  double scale;
  Widget widget;
  ScaleData(this.scale, this.widget);
}